#include "Arduino.h"
//#include "I2Cdev.h"
#include "Wire.h"
#include "WiFi.h"
#include "HTTPClient.h"
#include "ArduinoJson.h"
#include "ESP32Servo.h"

// Tracking Sensor
// for ESP32 microcontroller

// assign tracking sensor
const int Tracking_R = 33 ;
const int Tracking_L = 32 ;

// check track work
const int internal_LED = 2 ;

// assign mortor 1
const int MA1 = 16;     // Motor A1 14 -> 16
const int MA2 = 4;     // Motor A2 12 -> 4
const int PWM_A = 25;   // Speed Motor A 36 -> 25

// assign mortor 2
const int MB1 = 18;     // Motor B1 26 -> 18
const int MB2 = 19;     // Motor B2 27 -> 19
const int PWM_B = 26;   // Speed Motor B 39 -> 26

// Setting PWM prop
const int resolutionA = 8;
const int freqA = 30000;
const int pwmChannelA = 0;
const int pwmChannelB = 0;
const int resolutionB = 8;
const int freqB = 30000;


// Wifi Connect
const char* ssid = "FranKydeSU" ;
const char* password  = "512346789" ;
const char* urlget = "https://smart-bin-controller-server.herokuapp.com/";
const char* urlget_mov_0 = "https://smart-bin-controller-server.herokuapp.com/moving?state=0";
const char* urlget_left_0 = "";
const char* urlget_right_0 = "";

// assign get method
int isMoving ;
int _speed ;
int isTurnLeft;
int isTurnRight;
HTTPClient  http ;

StaticJsonDocument<2*JSON_OBJECT_SIZE(5)> JSONGet;

int SPEED = 100 ;  // Speed PWM 0 - 255

void WiFi_Connect() ;
void _get() ;
void Forward() ;
void Backward() ;
void turnLeft() ;
void turnRight() ;
void Stop() ;

void setup()
{
  Serial.begin(115200);
  //Setup tracking
  pinMode(Tracking_R, INPUT); //Tracking R
  pinMode(Tracking_L, INPUT);  //Tracking L
  pinMode(2, OUTPUT); 

  //Setup Channel A
  pinMode(MA1, OUTPUT); //Motor A1
  pinMode(MA2, OUTPUT); //Motor A2
  pinMode(PWM_A, OUTPUT); //Speed PWM Motor A

  //Setup Channel B
  pinMode(MB1, OUTPUT);  //Motor B1
  pinMode(MB2, OUTPUT);  //Motor B2
  pinMode(PWM_B, OUTPUT); //Speed PWM Motor B

  // config 
  ledcSetup(pwmChannelA, freqA, resolutionA);
  ledcSetup(pwmChannelB, freqB, resolutionB);

  // attach
  ledcAttachPin(PWM_A, pwmChannelA);
  ledcAttachPin(PWM_B, pwmChannelB);
  
  delay(4000) ; 

  // Connect to Wi-Fi
  WiFi.begin(ssid , password) ; 
  while(WiFi.status() != WL_CONNECTED) 
  {
    delay(1000) ;
    Serial.println("Connecting to WiFi...") ;
  }

  // Connected
  Serial.println("Connected to the WiFi network") ;
  
}

void loop()
{
  int val_R = digitalRead(Tracking_R) ;
  int val_L = digitalRead(Tracking_L) ;
  _get() ;
  int temp = 0;
  if(isMoving == 1)
  {
    Serial.println("Start Ving !!!") ;
    while(1)
    {
      if(temp++%2000 == 0)_get() ;
      if(_speed == 100)
      {
        SPEED = 170 ;
      }
      else if(_speed == 150)
      {
        SPEED = 200 ;
      }
      else if(_speed == 200)
      {
        SPEED = 255 ;
      }
      int val_R = digitalRead(Tracking_R) ;
      int val_L = digitalRead(Tracking_L) ;
      Serial.print("R = ") ;
      Serial.print(val_R) ;
      Serial.print(" , L = ") ;
      Serial.println(val_L) ; 

      if(val_R == 1 and val_L == 1)
      {
//        Serial.print("R = ") ;
//        Serial.print(val_R) ;
//        Serial.print(" , L = ") ;
//        Serial.println(val_L) ;
        Forward() ;
      }
      else if(val_R == 0 and val_L == 1)
      {
//        Serial.print("R = ") ;
//        Serial.print(val_R) ;
//        Serial.print(" , L = ") ;
//        Serial.println(val_L) ;
        turnRight() ;
      }
      else if(val_R == 1 and val_L == 0)
      {
//        Serial.print("R = ") ;
//        Serial.print(val_R) ;
//        Serial.print(" , L = ") ;
//        Serial.println(val_L) ;
        turnLeft() ;
      }
      else if(val_R == 0 and val_L == 0)
      {
//        Serial.print("R = ") ;
//        Serial.print(val_R) ;
//        Serial.print(" , L = ") ;
//        Serial.println(val_L) ;
        Stop() ;
//        delay(1000);
//        http.begin(urlget_mov_0) ;
//        http.GET() ;
//        isMoving = 0 ;
//        break ;
      }
      if(isMoving == 0)
      {
        Stop() ;
        http.begin(urlget_mov_0) ;
        http.GET() ;
        isMoving = 0 ;
        break ;        
      }
      Serial.print("SPEED = ") ;
      Serial.println(SPEED) ;
    }
  }
  delay(10) ;
}

void  WiFi_Connect() 
{
  WiFi.disconnect() ;
  WiFi.begin(ssid , password) ;

  while (WiFi.status() != WL_CONNECTED) {
    delay(1000) ;
    Serial.println("Connecting to WiFi...") ;
  }
  Serial.println("Conneccted to the WiFi network") ;
  Serial.print("IP Address: ") ;
  Serial.println(WiFi.localIP()) ;
}

void _get() {
  if (WiFi.status() == WL_CONNECTED) {
    // HTTPClient  http ;

    http.begin(urlget) ;

    int httpCode = http.GET() ;

    if (httpCode == HTTP_CODE_OK) {
      String payload = http.getString() ;
      DeserializationError err = deserializeJson(JSONGet , payload) ;
      if (err) 
      {
        Serial.print(F("deseaializeJson() failed with code: ")) ;
        Serial.println(err.c_str()) ;
      } 
      else 
      {
        isMoving = (int)JSONGet["moving_status"];
        _speed = (int)JSONGet["speed_status"];    
        isTurnLeft = (int)JSONGet["left"];
        isTurnRight = (int)JSONGet["right"];
        Serial.print("httpCode: ") ;
        Serial.println(httpCode);
        Serial.println(payload);
        Serial.println(isMoving) ;
        Serial.println(_speed) ;
      }
    } 
    else 
    {
      Serial.println(httpCode) ; 
      Serial.println("ERROR on HTTP Request") ;
    }
  } 
  else 
  {
    WiFi_Connect() ;
  }
}

void Forward()
{
  digitalWrite(MA1, LOW);
  digitalWrite(MA2, HIGH);
//  analogWrite(PWM_A, SPEED);
  ledcWrite(pwmChannelA, SPEED);

  digitalWrite(MB1, HIGH);
  digitalWrite(MB2, LOW);
  ledcWrite(pwmChannelB, SPEED);
//  analogWrite(PWM_B, SPEED);
  
}

void Backward ()
{
  digitalWrite(MA1, HIGH);
  digitalWrite(MA2, LOW);
//  analogWrite(PWM_A, SPEED);
  ledcWrite(pwmChannelA, SPEED);

  digitalWrite(MB1, LOW);
  digitalWrite(MB2, HIGH);
//  analogWrite(PWM_B, SPEED);
  ledcWrite(pwmChannelB, SPEED);
}

void turnLeft()
{

  digitalWrite(MB1, LOW);
  digitalWrite(MB2, LOW);
  ledcWrite(pwmChannelB, 0);
  
  digitalWrite(MA1, LOW);
  digitalWrite(MA2, HIGH);
  ledcWrite(pwmChannelA, 175);

}

void turnRight()
{
  digitalWrite(MA1, LOW);
  digitalWrite(MA2, LOW);
//  analogWrite(PWM_A, SPEED);
  ledcWrite(pwmChannelA, 0);

  digitalWrite(MB1, HIGH);
  digitalWrite(MB2, LOW);
  ledcWrite(pwmChannelB, 175);
//  analogWrite(PWM_B, SPEED);
  
}

void Stop()
{
  digitalWrite(MA1, LOW);
  digitalWrite(MA2, LOW);
//  analogWrite(PWM_A, 0);
  ledcWrite(pwmChannelA, 0);

  digitalWrite(MB1, LOW);
  digitalWrite(MB2, LOW);
  ledcWrite(pwmChannelB, 0);
//  analogWrite(PWM_B, 0);
}
